@@ -1,21 +1,29 @@
 from aerosandbox import Opti, ImplicitAnalysis
 from aerosandbox.geometry import *
 from aerosandbox.performance import OperatingPoint
-from aerosandbox.visualization import Figure3D
+from typing import Dict
 
 
 class LiftingLine(ImplicitAnalysis):
     """
-    An implicit analysis based on lifting line theory, with modifications for nonzero sweep and dihedral + multiple wings.
+    An implicit aerodynamics analysis based on lifting line theory, with modifications for nonzero sweep
+    and dihedral + multiple wings.
 
-    Key outputs:
-
-        * LiftingLine.CL
-        * LiftingLine.CD
-        * LiftingLine.Cm
-        * LiftingLine.lift_force
-        * LiftingLine.drag_force
+    Nonlinear, and includes viscous effects based on 2D data.
 
+    Usage example:
+        >>>analysis = asb.LiftingLine(
+        >>>    airplane=my_airplane,
+        >>>    op_point=asb.OperatingPoint(
+        >>>        velocity=100, # m/s
+        >>>        alpha=5, # deg
+        >>>        beta=4, # deg
+        >>>        p=0.01, # rad/sec
+        >>>        q=0.02, # rad/sec
+        >>>        r=0.03, # rad/sec
+        >>>    )
+        >>>)
+        >>>outputs = analysis.run()
     """
 
     @ImplicitAnalysis.initialize
@@ -24,8 +32,8 @@ class LiftingLine(ImplicitAnalysis):
                  op_point: OperatingPoint,
                  run_symmetric_if_possible=True,
                  verbose=True,
-                 default_n_spanwise_panels = 8, # TODO document
-                 default_spanwise_spacing = "cosine" # TODO document
+                 spanwise_resolution=8,  # TODO document
+                 spanwise_spacing="cosine"  # TODO document
                  ):
         """
         Initializes and conducts a LiftingLine analysis.
@@ -47,14 +55,15 @@ class LiftingLine(ImplicitAnalysis):
                 If not provided, creates and solves the governing equations in a new instance.
 
         """
+        super().__init__()
 
         ### Initialize
         self.airplane = airplane
         self.op_point = op_point
         self.run_symmetric_if_possible = run_symmetric_if_possible
         self.verbose = verbose
-        self.default_n_spanwise_panels = default_n_spanwise_panels
-        self.default_spanwise_spacing = default_spanwise_spacing
+        self.spanwise_resolution = spanwise_resolution
+        self.spanwise_spacing = spanwise_spacing
 
         ### Determine whether you should run the problem as symmetric
         self.run_symmetric = False
@@ -66,17 +75,12 @@ class LiftingLine(ImplicitAnalysis):
                         self.op_point.r == 0 and
                         self.airplane.is_entirely_symmetric()
                 )
-            except RuntimeError: # Required because beta, p, r, etc. may be non-numeric (e.g. opti variables)
+            except RuntimeError:  # Required because beta, p, r, etc. may be non-numeric (e.g. opti variables)
                 pass
 
-        self._make_panels()
-        self._setup_geometry()
-        self._setup_operating_point()
-        self._calculate_vortex_strengths()
-        self._calculate_forces()
+    def run(self) -> Dict:
 
-    def _make_panels(self):
-        # Creates self.panel_coordinates_structured_list and self.wing_mcl_normals.
+        ### Make Panels
 
         if self.verbose:
             print("Meshing...")
@@ -90,12 +94,29 @@ class LiftingLine(ImplicitAnalysis):
         Cm_functions = []
         wing_id = []
 
-        for wing_index, wing in enumerate(self.airplane.wings):  # Iterate through wings
+        for wing in self.airplane.wings:  # Iterate through wings
             for inner_xsec, outer_xsec in zip(
                     wing.xsecs[:-1],
                     wing.xsecs[1:]
             ):  # Iterate through pairs of wing cross sections
 
+                wing_section = Wing(
+                    xyz_le=wing.xyz_le,
+                    xsecs=[
+                        inner_xsec,
+                        outer_xsec
+                    ],
+                    symmetric=wing.symmetric
+                )
+
+                points, faces = wing_section.mesh_thin_surface(
+                    method="quad",
+                    chordwise_resolution=1,
+                    spanwise_resolution=self.spanwise_resolution,
+                    spanwise_spacing=spanwise_spacing,
+                    add_camber=False
+                )
+
                 # Find the corners
                 inner_xsec_xyz_le = inner_xsec.xyz_le + wing.xyz_le
                 inner_xsec_xyz_te = inner_xsec.xyz_te() + wing.xyz_le
@@ -106,7 +127,7 @@ class LiftingLine(ImplicitAnalysis):
                 try:
                     n_spanwise_panels = inner_xsec.spanwise_panels
                 except AttributeError:
-                    n_spanwise_panels = self.default_n_spanwise_panels
+                    n_spanwise_panels = self.spanwise_resolution
 
                 n_spanwise_coordinates = n_spanwise_panels + 1
 
@@ -114,7 +135,7 @@ class LiftingLine(ImplicitAnalysis):
                 try:
                     spanwise_spacing = inner_xsec.spanwise_spacing
                 except AttributeError:
-                    spanwise_spacing = self.default_spanwise_spacing
+                    spanwise_spacing = self.spanwise_spacing
 
                 if spanwise_spacing == 'uniform':
                     nondim_spanwise_coordinates = np.linspace(0, 1, n_spanwise_coordinates)
@@ -154,7 +175,8 @@ class LiftingLine(ImplicitAnalysis):
                         lambda alpha, Re, mach,
                                inner_xsec=inner_xsec,
                                outer_xsec=outer_xsec,
-                               nondim_spanwise_coordinate=(nondim_spanwise_coordinate_inner + nondim_spanwise_coordinate_outer) / 2,
+                               nondim_spanwise_coordinate=(
+                                                                  nondim_spanwise_coordinate_inner + nondim_spanwise_coordinate_outer) / 2,
                         : (
                                 inner_xsec.airfoil.CL_function(
                                     alpha=alpha, Re=Re, mach=mach,
@@ -170,7 +192,8 @@ class LiftingLine(ImplicitAnalysis):
                         lambda alpha, Re, mach,
                                inner_xsec=inner_xsec,
                                outer_xsec=outer_xsec,
-                               nondim_spanwise_coordinate=(nondim_spanwise_coordinate_inner + nondim_spanwise_coordinate_outer) / 2,
+                               nondim_spanwise_coordinate=(
+                                                                  nondim_spanwise_coordinate_inner + nondim_spanwise_coordinate_outer) / 2,
                         : (
                                 inner_xsec.airfoil.CDp_function(
                                     alpha=alpha, Re=Re, mach=mach,
@@ -186,7 +209,8 @@ class LiftingLine(ImplicitAnalysis):
                         lambda alpha, Re, mach,
                                inner_xsec=inner_xsec,
                                outer_xsec=outer_xsec,
-                               nondim_spanwise_coordinate=(nondim_spanwise_coordinate_inner + nondim_spanwise_coordinate_outer) / 2,
+                               nondim_spanwise_coordinate=(
+                                                                  nondim_spanwise_coordinate_inner + nondim_spanwise_coordinate_outer) / 2,
                         : (
                                 inner_xsec.airfoil.Cm_function(
                                     alpha=alpha, Re=Re, mach=mach,
@@ -211,7 +235,8 @@ class LiftingLine(ImplicitAnalysis):
                             lambda alpha, Re, mach,
                                    inner_xsec=inner_xsec,
                                    outer_xsec=outer_xsec,
-                                   nondim_spanwise_coordinate=(nondim_spanwise_coordinate_inner + nondim_spanwise_coordinate_outer) / 2,
+                                   nondim_spanwise_coordinate=(
+                                                                      nondim_spanwise_coordinate_inner + nondim_spanwise_coordinate_outer) / 2,
                             : (
                                     inner_xsec.airfoil.CL_function(
                                         alpha=alpha, Re=Re, mach=mach,
@@ -231,7 +256,8 @@ class LiftingLine(ImplicitAnalysis):
                             lambda alpha, Re, mach,
                                    inner_xsec=inner_xsec,
                                    outer_xsec=outer_xsec,
-                                   nondim_spanwise_coordinate=(nondim_spanwise_coordinate_inner + nondim_spanwise_coordinate_outer) / 2,
+                                   nondim_spanwise_coordinate=(
+                                                                      nondim_spanwise_coordinate_inner + nondim_spanwise_coordinate_outer) / 2,
                             : (
                                     inner_xsec.airfoil.CDp_function(
                                         alpha=alpha, Re=Re, mach=mach,
@@ -251,7 +277,8 @@ class LiftingLine(ImplicitAnalysis):
                             lambda alpha, Re, mach,
                                    inner_xsec=inner_xsec,
                                    outer_xsec=outer_xsec,
-                                   nondim_spanwise_coordinate=(nondim_spanwise_coordinate_inner + nondim_spanwise_coordinate_outer) / 2,
+                                   nondim_spanwise_coordinate=(
+                                                                      nondim_spanwise_coordinate_inner + nondim_spanwise_coordinate_outer) / 2,
                             : (
                                     inner_xsec.airfoil.Cm_function(
                                         alpha=alpha, Re=Re, mach=mach,
@@ -339,7 +366,7 @@ class LiftingLine(ImplicitAnalysis):
 
             fuse_centerline_points.append(
                 this_fuse_centerline_points)  # TODO handle fuselage symmetry (non-symmetric problem)
-            fuse_radii.append(this_fuse_radii) # TODO resume here
+            fuse_radii.append(this_fuse_radii)  # TODO resume here
             if (not self.run_symmetric) and fuse.symmetric:
                 fuse_centerline_points.append(reflect_over_XZ_plane(this_fuse_centerline_points))
                 fuse_radii.append(this_fuse_radii)
@@ -918,6 +945,9 @@ class LiftingLine(ImplicitAnalysis):
         To solve an AeroProblem, use opti.solve(). To substitute a solved solution, use ap = ap.substitute_solution(sol).
         :return:
         """
+
+        # TODO rewrite me
+
         if self.verbose:
             print("Drawing...")
 
